<?php defined('SYSPATH') OR die('No direct script access.');

abstract class Controller_Handler_Api extends Controller
{
    /** @var  Validation */
    protected $get;

    /** @var  Validation */
    protected $post;

    /** @var  User */
    protected $user;

    /** @var  Token */
    protected $token;

    public function before() {
        // todo: var $user should be defined here
        //    either by GET param (token),
        //    or on the session variables.

        $this->token = $this->request->query('token');
        if ( $this->token != '' )
        {
            $ok = User::instance()->signInByToken($this->token);
            if (!$ok) $this->fail();
        }

        $this->user = User::instance();

        if (!$this->user->isAuthorized())
            $this->fail();

        $this->get = Validation::factory(Arr::map('trim', $_GET));
        $this->post = Validation::factory(Arr::map('trim', $_POST));
    }

    public function execute() {
        // Execute the "before action" method
        $this->before();

        // Determine the action to use
        $action = 'action_' . $this->request->action();

        // If the action doesn't exist, it's a 404
        if (!method_exists($this, $action)) {
            throw HTTP_API_Exception::factory(404,
                'The requested URL :uri was not found on this server.',
                [':uri' => $this->request->uri()]
            )->request($this->request);
        }

        try {
            // Execute the action itself
            $data = $this->{$action}();
            $answer = ['response' => $data ?: ''];
            $json_answer = json_encode($answer, JSON_UNESCAPED_UNICODE);
        } catch (HTTP_API_Exception $e) { // Let HTTP errors go throw away
            throw $e;
        } catch (Exception $e) {
            // todo: improve error capturing
            while ($e->getPrevious())
                $e = $e->getPrevious();

            $answer = [
                'error' => [
                    'code'    => (int) $e->getCode(),
                    'message' => (string) $e->getMessage(),
//                  'request' => $this->request->params    // like http://vk.com/dev/captcha_error
                ]
            ];
            $json_answer = json_encode($answer, JSON_UNESCAPED_UNICODE);
        }

        // Json API
        if (!$this->request->is_ajax())
            $json_answer = $this->indent($json_answer);
        $this->response->body($json_answer);

        // Execute the "after action" method
        $this->after();

        // Return the response
        return $this->response;
    }

    /**
     * Indents a flat JSON string to make it more human-readable.
     * @param string $json The original JSON string to process.
     * @return string Indented version of the original JSON string.
     */
    private function &indent(&$json) {
        $result = '';
        $pos = 0;
        $strLen = strlen($json);
        $indentStr = '  ';
        $newLine = "\n";
        $prevChar = '';
        $outOfQuotes = true;

        for ($i = 0; $i <= $strLen; $i++) {
            // Grab the next character in the string.
            $char = substr($json, $i, 1);

            // Are we inside a quoted string?
            if ($char == '"' && $prevChar != '\\') {
                $outOfQuotes = !$outOfQuotes;

                // If this character is the end of an element,
                // output a new line and indent the next line.
            } else if (($char == '}' || $char == ']') && $outOfQuotes) {
                $result .= $newLine;
                $pos--;
                for ($j = 0; $j < $pos; $j++) {
                    $result .= $indentStr;
                }
            }

            // Add the character to the result string.
            $result .= $char;

            // If the last character was the beginning of an element,
            // output a new line and indent the next line.
            if (($char == ',' || $char == '{' || $char == '[') && $outOfQuotes) {
                $result .= $newLine;
                if ($char == '{' || $char == '[') {
                    $pos++;
                }

                for ($j = 0; $j < $pos; $j++) {
                    $result .= $indentStr;
                }
            }

            $prevChar = $char;
        }

        return $result;
    }

    protected function fail() {
        throw HTTP_API_Exception::factory(403,
            'Token is broken.',
            [':uri' => $this->request->uri()]
        )->request($this->request);
    }

    protected function notFoundError($message) {
        throw HTTP_API_Exception::factory(404,
            $message,
            [':uri' => $this->request->uri()]
        )->request($this->request);
    }

    protected function badRequestError($message) {
        throw HTTP_API_Exception::factory(400,
            $message,
            [':uri' => $this->request->uri()]
        )->request($this->request);
    }
}