<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Handler_Map extends Controller_Handler
{
    /** @var  Model_User_Teacher */
    protected $user;


    public function before() {
        parent::before();

        $this->user->checkAccess(User::RIGHTS_TEACHER);
    }


    public function action_ChangeDisciplineSubject() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->changeDisciplineSubject($this->user->TeacherID, $_POST['SubjectID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeStatusBonusModule() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);


        if ($this->post['BonusRate'] === "true") {
            $failed = $map->addModuleBonus($this->user->TeacherID);
            $response['action'] = 'add';
        } else {
            $failed = $map->deleteModuleBonus($this->user->TeacherID);
            $response['action'] = 'delete';
        }

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeDisciplineGrade() {
        $failed = Model_Discipline::load($_POST['DisciplineID'])
            ->changeGrade($this->user->TeacherID, $_POST['GradeID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeDisciplineControl() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);
        $result = $map->changeDisciplineControl($this->user->TeacherID, $_POST['Control']);

        $response['success'] = $result;
        $this->response->body(json_encode($response));
    }

    public function action_ChangeDisciplineHours() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->changeDisciplineHours($this->user->TeacherID, $_POST['Hours'], $_POST['Type']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_GetSubjectsList() {
        $this->response->body(json_encode(
            Model_Faculty::with($_POST['FacultyID'])->getSubjects()));
    }


    // Structure stuff

    public function action_addModule() {
        $map = Model_Map::of(Model_Discipline::load($_POST['id']));
        
        $module_id = $map->addModule($this->user->TeacherID);
        if ($module_id < 0) $this->fail();
        
        $submodule = $map->addSubmodule($this->user->TeacherID, $module_id);
        if ($submodule < 0) $this->fail();
        
        $this->response->body(json_encode(['module' => $module_id, 'submodule' => $submodule]));
    }

    public function action_addSubmodule() {
        $id = Model_Map::addSubmodule($this->user->TeacherID, $_POST['module']);
        $id < 0 ? $this->fail() : $this->response->body(json_encode(['submodule' => $id]));
    }

    public function action_setModuleName() {
        $_POST['type'] == 'module'
            ? $err = Model_Map::setModuleName($this->user->TeacherID, $_POST['id'], $_POST['name'])
            : $err = Model_Map::setSubmoduleName($this->user->TeacherID, $_POST['id'], $_POST['name']);

        if ($err) $this->fail();
    }

    public function action_swapModules() {
        $_POST['type'] == 'module'
            ? $err = Model_Map::swapModules($this->user->TeacherID, $_POST['ID1'], $_POST['ID2'])
            : $err = Model_Map::swapSubmodules($this->user->TeacherID, $_POST['ID1'], $_POST['ID2']);

        if ($err) $this->fail();
    }

    public function action_deleteModule() {
        $_POST['type'] == 'module'
            ? $err = Model_Map::deleteModule($this->user->TeacherID, $_POST['id'])
            : $err = Model_Map::deleteSubmodule($this->user->TeacherID, $_POST['id']);
        
        if ($err) $this->fail();
    }

    public function action_setSubmoduleMaxAndControl() {
        if ($err = Model_Map::setSubmoduleMaxAndControl($this->user->TeacherID, $_POST['id'], $_POST['maxRate'], $_POST['control']))
            $this->fail();
    }


    public function action_BindGroup() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $code = $map->bindGroup($this->user->TeacherID, $_POST['GroupID']);

        $response = ['success' => (0 == $code), 'code' => $code,];
        $this->response->body(json_encode($response));
    }

    public function action_UnbindGroup() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->unbindGroup($this->user->TeacherID, $_POST['GroupID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_BindStudent() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->bindStudent($this->user->TeacherID, $_POST['StudentID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_UnbindStudent() {
        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        $failed = $map->unbindStudent($this->user->TeacherID, $_POST['StudentID']);

        $response['success'] = !$failed;
        $this->response->body(json_encode($response));
    }

    public function action_GetGroups() {
        $faculty = Model_Faculty::with($_POST['FacultyID']);
        $groups = $faculty->getGroups($_POST['GradeID']);
        $this->response->body(json_encode($groups));
    }

    public function action_GetGrades() {
        $grades = Model_Grades::loadAll()->as_array();
        $this->response->body(json_encode($grades));
    }

    public function action_SearchStudents() {
        $result = Model_Students::search($_POST['GradeID'], $_POST['GroupID'], $_POST['FacultyID'], $_POST['Name'], $_POST['DisciplineID']);
        $this->response->body(json_encode($result));
    }

    public function action_GetDepartments() {
        $faculty = Model_Faculty::with($this->post['FacultyID']);
        $departments = $faculty->getDepartments();
        $this->response->body(json_encode($departments));
    }

    public function action_SearchTeachers() {
        $teachers = Model_Teachers::search($_POST['Last'], $_POST['FacultyID'], $_POST['DepartmentID'], $_POST['DisciplineID']);
        $this->response->body(json_encode($teachers));
    }
}
