<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Handler_Students extends Controller_Handler
{
    public function before() {
        parent::before();

        $this->user->checkAccess(User::RIGHTS_ADMIN | User::RIGHTS_DEAN);
    }

    public function action_create() {
//        $this->user->checkAccess(User::RIGHTS_ADMIN);

        $student = Model_Student::make()
            ->faculty($this->post['facultyID'])
            ->grade($this->post['gradeNum'])
            ->group($this->post['groupNum'])
            ->firstName($this->post['firstName'])
            ->secondName($this->post['secondName'])
            ->lastName($this->post['lastName'])
            ->create();

        $res = ['ID' => $student->ID, 'Code' => $student->ActivationCode];
        $this->response->body(json_encode($res));
    }

    public function action_search() {
        $facultyID = $this->post['FacultyID'] ?: $this->user->Faculty->ID;
        $gradeID = $this->post['GradeID'];
        $groupID = $this->post['GroupID'];
        $name = $this->post['Name'];
        $semester = $this->post['semesterID'];

        $p = preg_split('/\s+/', $name);
        $list = Model_Students::searchStudentsByName($p, $facultyID, $gradeID, $groupID, $semester);

        $this->response->body(json_encode($list));
    }

    public function action_getByGroup()
    {
        $semesterID = $this->post['semesterID'];
        $groupID = $this->post['studyGroupID'];
        $this->response->body(json_encode(Model_Students::ofGroup($groupID, $semesterID)));
    }

    public function action_getGroups() {
        $facultyID = $this->post['facultyID'];
        $gradeID = $this->post['gradeNum'];

        $groupsHandled = [];
        if (($facultyID && $gradeID) != 0) {
            $groups = Model_Faculty::with($facultyID)->getGroups($gradeID);

            foreach ($groups as $row)
                $groupsHandled[] = [
                    'ID' => $row['ID'],
                    'Num' => $row['GroupNum'],
                    'SpecName' => is_null($row['SpecName']) ? 'Без специализации' : $row['SpecName'],
                    'SpecAbbr' => is_null($row['SpecAbbr']) ? '' : $row['SpecAbbr'],
                ];
        }
        $this->response->body(json_encode($groupsHandled));
    }

    private function groupByID($array)
    {
        $result = [];
        foreach($array as $key => $value)
            $result[] = $value['ID'];
        return $result;
    }

    public function action_transfer() {
        $this->user->checkAccess(User::RIGHTS_ADMIN | User::RIGHTS_DEAN);
        $semesterID = (int) $this->post['semesterID'];
        $groupID = (int) $this->post['studyGroupID'];
        $transferList = json_decode($this->post['transferList']);

        // Отметаем всех студентов, которые уже есть в группе
        $group = $this->groupByID(Model_Students::ofGroup($groupID, $semesterID));
        $transferList = array_diff($transferList, $group);

        foreach($transferList as $ID)
        {
            // TODO: review the name of this method
            Model_Student::load($ID)->returnFromAcademicLeave($groupID, $semesterID);
        }

        $this->action_getByGroup();
    }
}
