<?php defined('SYSPATH') or die('No direct script access.');

/**
 * @property-read $ID int
 */
class Model_Faculty extends Model
{
    private $ID;

    public function __get($name) {
        if ($name === 'ID')
            return $this->ID;
        throw new RuntimeException();
    }

    public static function with($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('ID is incorrect!');

        $f = new self();
        $f->ID = (int) $id;
        return $f;
    }


    /**
     * @param $semesterID int
     * @return Database_Result
     */
    public function getMilestone($semesterID = null) {
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $sql = 'SELECT `GetMilestone`(:faculty, :semester) AS `Num`';
        return DB::query(Database::SELECT, $sql)
            ->param(':faculty', $this->ID)
            ->param(':semester', $semesterID)
            ->execute()[0]['Num'];
    }

    /**
     * @param $stage      int
     * @param $semesterID int
     */
    public function setMilestoneForCredits($stage, $semesterID = null) {
        if ($stage >= 0 && $stage <= 3)
            throw new LogicException('Milestone argument is incorrect!');

        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $sql = 'SELECT `RestrictAfterMilestoneForCredits`(:facultyID, :milestone, :semesterID)';
        DB::query(Database::SELECT, $sql)
            ->parameters([
                ':facultyID'  => $this->ID,
                ':milestone'  => $stage,
                ':semesterID' => $semesterID,
            ])->execute();
    }


    /**
     * @param  $semesterID int
     * @return Model_Discipline[]
     */
    public function getDisciplines($semesterID = null) {
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $sql = 'CALL `GetDisciplines`(:id, :semesterID)';
        $query = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semesterID', $semesterID)
            ->execute();

        $list = [];
        foreach ($query as $data)
            $list[] = new Model_Discipline($data, true);

        return $list;
    }

    /**
     * @param $grade        int
     * @param $semesterID   int
     * @return array
     */
    public function getGroups($grade, $semesterID = null) {
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $sql = 'CALL `GetGroups`(:grade, :id)';
        return DB::query(Database::SELECT, $sql)
            ->param(':grade', (int) $grade)
            ->param(':id', $this->ID)
            ->execute()->as_array();
    }

    public function getSubjects() {
        $sql = 'CALL `GetSubjects`(:faculty)';
        return DB::query(Database::SELECT, $sql)
            ->param(':faculty', $this->ID)
            ->execute()->as_array();
    }

    public function getDepartments() {
        $sql = 'CALL `GetDepartments`(:faculty)';
        return DB::query(Database::SELECT, $sql)
            ->param(':faculty', $this->ID)
            ->execute()->as_array();
    }

    // todo: implement json_serialize
}
